package geometry;

import org.opencv.core.Point;

public class Ray {

	private double x0;
	private double y0;
	private double dirX;
	private double dirY;

	public Ray(Point point1, double dirX, double dirY) {
		setX0(point1.x);
		setY0(point1.y);
		
		setDirX(dirX);
		setDirY(dirY);
	}
	
	public Ray(Point point1, Point point2) {
		setX0(point1.x);
		setY0(point1.y);

		setDirX(point2.x - x0);
		setDirY(point2.y - y0);
	}

	public Point calcIntersectionPoint(Ray toCheck) {
		double denominator = toCheck.dirX * this.dirY - toCheck.dirY * this.dirX;

		if (denominator == 0) {
			return null;
		}

		double alpha = (this.x0 * this.dirY - this.y0 * this.dirX - toCheck.x0 * this.dirY
				+ toCheck.y0 * this.dirX) / denominator;

		Point result = new Point(toCheck.x0 + alpha * toCheck.dirX, toCheck.y0 + alpha * toCheck.dirY);

		return result;
	}

	public double getX0() {
		return x0;
	}

	public void setX0(double x0) {
		this.x0 = x0;
	}

	public double getY0() {
		return y0;
	}

	public void setY0(double y0) {
		this.y0 = y0;
	}

	public double getDirX() {
		return dirX;
	}

	public void setDirX(double dirX) {
		this.dirX = dirX;
	}

	public double getDirY() {
		return dirY;
	}

	public void setDirY(double dirY) {
		this.dirY = dirY;
	}
}