package geometry;

import org.opencv.core.MatOfPoint;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.RotatedRect;
import org.opencv.imgproc.Imgproc;

import imageProcessing.ImageProcessor;
import linePP.data.Segment;

public class StraightLine {
	private Point tl;
	private Point bl;
	
	private double slope;
	private double yIntersect;
	private boolean slopIsInf;
	
	public StraightLine(double x0, double y0, double vw, double vy, double top, double bottom) {
		double tlx = x0 + vw * (top - y0) / vy;
		double blx = x0 + vw * (bottom - y0) / vy;
		
		Point tl = new Point(tlx, top);
		Point bl = new Point(blx, bottom);
		
		setTl(tl);
		setBl(bl);
		setSlope(calcSlope(tl, bl));
		setyIntersect(calcYIntersect(tl, slope));
	}
	
	public StraightLine(Segment segment) {
		RotatedRect rotRect = Imgproc.minAreaRect(new MatOfPoint2f(segment.getRegion().toArray()));
		Point[] points = new Point[4];
		rotRect.points(points);
		
		//{tl, tr, br, bl}
		Point[] orderedPoints = ImageProcessor.findLinePointOrder(new MatOfPoint(points));
		Point tl = orderedPoints[0];
		Point bl = orderedPoints[3];
		
		setTl(tl);
		setBl(bl);
		setSlope(calcSlope(tl, bl));
		setyIntersect(calcYIntersect(tl, slope));
	}
	
	public double calcX(double y) {
		double x = (y - yIntersect) / slope;
		
		return x;
	}
	
	private double calcYIntersect(Point tl, double slope) {
		double yIntersect = tl.y - slope * tl.x;
		
		return yIntersect;
	}
	
	private double calcSlope(Point tl, Point bl) {
		if(bl.x - tl.x == 0) {
			setSlopIsInf(true);
		}
		double slope = (bl.y - tl.y) / (bl.x - tl.x);
		
		return slope;
	}

	public double getSlope() {
		return slope;
	}

	public void setSlope(double slope) {
		this.slope = slope;
	}

	public double getyIntersect() {
		return yIntersect;
	}

	public void setyIntersect(double yIntersect) {
		this.yIntersect = yIntersect;
	}

	public boolean isSlopIsInf() {
		return slopIsInf;
	}

	public void setSlopIsInf(boolean slopIsInf) {
		this.slopIsInf = slopIsInf;
	}

	public Point getTl() {
		return tl;
	}

	public void setTl(Point tl) {
		this.tl = tl;
	}

	public Point getBl() {
		return bl;
	}

	public void setBl(Point bl) {
		this.bl = bl;
	}	
}