package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.opencv.core.Mat;

import dataManagement.DataManager;
import dataManagement.Page;
import gui.drawing.BackgroundPanel;
import gui.initParameters.PnlPages;
import gui.initParameters.PnlProcessParameters;
import gui.initParameters.PnlRegionParameters;
import gui.postProcessing.PnlPostProcessing;
import gui.postProcessing.ResultsPanel;
import importExport.SettingsReader;
import importExport.SettingsWriter;
import linePP.DiaCalcAll;
import linePP.calc.LinePP;
import linePP.edit.LineEditor;
import net.miginfocom.swing.MigLayout;
import segmentation.ImageProcessor;
import segmentation.parameters.Parameters;

@SuppressWarnings("serial")
public class Gui extends JFrame {

	// private static final String LAYOUT_DEBUG = "debug";
	private static final String LAYOUT_DEBUG = "";

	private int verticalResolution;

	private GuiManager guiManager;
	private DataManager dataManager;

	private BackgroundPanel pnlBackground;
	private ResultsPanel pnlResults;

	private JPanel pnlImage;
	private JPanel pnlControls;
	private JPanel pnlRadioButtons;
	private PnlPages pnlPages;
	private JPanel pnlLinePostProc;

	private PnlRegionParameters pnlRegionParameters;
	private PnlProcessParameters pnlProcessParameters;
	private PnlPostProcessing pnlPostProcessing;

	private JRadioButton rbOriginal;
	private JRadioButton rbBinary;
	private JRadioButton rbResult;
	private JRadioButton rbPostProc;

	private JButton btnDoSomething;
	private JButton btnSegmentAll;
	private JButton btnPrevious;
	private JButton btnNext;
	private JButton btnSaveSettings;
	private JButton btnLoadSettings;
	private JButton btnLppEdit;
	private JButton btnLppCalcAll;
	private JButton btnLppCalc;
	private JButton btnLPPAuto;

	private JCheckBox cbSegOnSwitch;
	private JCheckBox cbFullAuto;

	public Gui(ArrayList<Page> pages, String outputFolder, int verticalResolution) {
		super();

		setVerticalResolution(verticalResolution);
		pages.get(0).initPage(verticalResolution);
		setDataManager(new DataManager(pages));
		setGuiManager(new GuiManager(this));
		guiManager.setOutputFolder(outputFolder);

		createWidgets(pages.get(0).getResized());
		addWidgets();
		addListener();

		setPnlPostProcessing(pnlPostProcessing);

		// guiManager.loadPage(guiManager.getCurrentPage());

		pack();
		setLocationRelativeTo(null);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);

		SettingsReader.loadSettings(guiManager.getOutputFolder(), guiManager);

		guiManager.initFirstSegResult();
	}

	public void showImage(Mat image) {
		switchToBackgroundPanel();
		Mat resized = ImageProcessor.resize(image, verticalResolution);
		pnlBackground.updateImage(resized, false);
	}

	private void addListener() {
		btnLppCalc.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				LinePP linePP = new LinePP(guiManager.getOutputFolder());
				linePP.calcLines(guiManager.getCurrentPage());
			}
		});

		btnLppCalcAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				new DiaCalcAll(guiManager.getOutputFolder());
			}
		});

		btnLPPAuto.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				new LineEditor(guiManager.getCurrentPage(), verticalResolution, true);
			}
		});

		btnLppEdit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				new LineEditor(guiManager.getCurrentPage(), verticalResolution, false);
			}
		});

		btnSaveSettings.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				SettingsWriter.saveSettings(guiManager.getOutputFolder(),
						new Parameters(guiManager.getRegionManager(), guiManager.getGui()));
			}
		});

		btnLoadSettings.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				SettingsReader.loadSettings(guiManager.getOutputFolder(), guiManager);
			}
		});

		btnNext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.showNextPage();
			}
		});

		btnPrevious.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.showPreviousPage();
			}
		});

		btnDoSomething.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.segmentCurrentPage();
				rbBinary.setEnabled(true);
				rbResult.setEnabled(true);
				rbPostProc.setEnabled(true);
				rbResult.setSelected(true);
				rbBinary.setSelected(false);
				rbOriginal.setSelected(false);
			}
		});

		btnSegmentAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (btnSegmentAll.getText().equals("Segment All")) {
					btnSegmentAll.setText("Stop");
					guiManager.segmentAllPages();
				} else {
					btnSegmentAll.setText("Segment All");
					guiManager.setKeepSegmenting(false);
				}
			}
		});

		rbOriginal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(true);
				rbBinary.setSelected(false);
				rbResult.setSelected(false);
				rbPostProc.setSelected(false);

				switchToBackgroundPanel();
				showImage(dataManager.getCurrentPage().getResized());
			}
		});

		rbBinary.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(false);
				rbBinary.setSelected(true);
				rbResult.setSelected(false);
				rbPostProc.setSelected(false);

				switchToBackgroundPanel();
				guiManager.calcAndShowBinary();
			}
		});

		rbResult.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(false);
				rbBinary.setSelected(false);
				rbResult.setSelected(true);
				rbPostProc.setSelected(false);

				switchToBackgroundPanel();
				showImage(dataManager.getCurrentPage().getResult());
			}
		});

		rbPostProc.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(false);
				rbBinary.setSelected(false);
				rbResult.setSelected(false);
				rbPostProc.setSelected(true);

				switchToResultsPanel();
			}
		});
	}

	@SuppressWarnings("deprecation")
	public void switchToResultsPanel() {
		if (pnlBackground != null) {
			pnlImage.remove(pnlBackground);
		}
		pnlImage.remove(pnlRadioButtons);

		pnlResults = new ResultsPanel(dataManager.getCurrentPage().getOriginal(),
				dataManager.getCurrentPage().getResized(), dataManager.getCurrentPage().getSegmentationResult());
		pnlResults.setBounds(7, 7, pnlResults.bounds().width, pnlResults.bounds().height); // TODO
		// wtf??

		pnlImage.add(pnlResults, "wrap");
		pnlImage.add(pnlRadioButtons, "center");
		repaint();
	}

	public void switchToBackgroundPanel() {
		if (pnlResults != null) {
			pnlImage.remove(pnlResults);
		}
		pnlImage.remove(pnlRadioButtons);
		pnlImage.add(pnlBackground, "wrap");
		pnlImage.add(pnlRadioButtons, "center");
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(LAYOUT_DEBUG);
		setLayout(layout);
		add(pnlImage);
		add(pnlControls);

		MigLayout layoutImage = new MigLayout(LAYOUT_DEBUG);
		pnlImage.setLayout(layoutImage);

		MigLayout layoutButtons = new MigLayout(LAYOUT_DEBUG);
		pnlRadioButtons.setLayout(layoutButtons);

		pnlRadioButtons.add(btnPrevious, "sg 1, center");
		pnlRadioButtons.add(btnNext, "sg 1, center");

		// pnlRadioButtons.add(btnPrevious, "sg 1");
		// pnlRadioButtons.add(btnNext, "sg 1");

		pnlRadioButtons.add(rbOriginal);
		pnlRadioButtons.add(rbBinary);
		pnlRadioButtons.add(rbResult);
		pnlRadioButtons.add(rbPostProc);

		pnlImage.add(pnlBackground, "wrap");
		pnlImage.add(pnlRadioButtons, "center");

		MigLayout layoutControls = new MigLayout(LAYOUT_DEBUG);
		pnlControls.setLayout(layoutControls);

		MigLayout layoutLinePP = new MigLayout(LAYOUT_DEBUG);
		pnlLinePostProc.setLayout(layoutLinePP);
		pnlLinePostProc.add(btnLppEdit, "sgx 1, pushx, growx");
		pnlLinePostProc.add(btnLppCalc, "sgx 1, pushx, growx");
		pnlLinePostProc.add(btnLppCalcAll, "sgx 1, pushx, growx, wrap");
		pnlLinePostProc.add(btnLPPAuto, "split, span, pushx, growx");

		pnlControls.add(pnlRegionParameters, "sgx 1, wrap");
		pnlControls.add(pnlProcessParameters, "sgx 1, wrap");
		pnlControls.add(btnSaveSettings, "split, pushx, growx");
		pnlControls.add(btnLoadSettings, "split, pushx, growx, wrap");
		pnlControls.add(pnlPostProcessing, "sgx 1, wrap");
		pnlControls.add(pnlLinePostProc, "sgx 1, wrap");
		pnlControls.add(pnlPages, "sgx 1, wrap");
		pnlControls.add(btnDoSomething, "sg 2, split, pushx, growx, sg 2");
		pnlControls.add(cbSegOnSwitch, "sg 3, wrap");
		pnlControls.add(btnSegmentAll, "sg 2, split, pushx, growx, sg 2");
		pnlControls.add(cbFullAuto, "sg 3, wrap");
	}

	private void createWidgets(Mat input) {
		setContentPane(new JPanel());

		pnlBackground = new BackgroundPanel(input, guiManager.getRegionManager());

		pnlImage = new JPanel();
		pnlControls = new JPanel();
		pnlPages = new PnlPages(guiManager, LAYOUT_DEBUG);
		pnlRadioButtons = new JPanel();

		pnlRegionParameters = new PnlRegionParameters(this, LAYOUT_DEBUG);
		pnlProcessParameters = new PnlProcessParameters(guiManager, LAYOUT_DEBUG);
		pnlPostProcessing = new PnlPostProcessing(this);

		pnlLinePostProc = new JPanel();
		pnlLinePostProc.setBorder(BorderFactory.createTitledBorder("Line Post Processing"));

		rbOriginal = new JRadioButton("Original");
		rbOriginal.setSelected(true);
		rbBinary = new JRadioButton("Binary");
		rbBinary.setEnabled(true);
		rbResult = new JRadioButton("Result");
		rbResult.setEnabled(false);
		rbPostProc = new JRadioButton("PostProc");
		rbPostProc.setEnabled(false);

		btnPrevious = new JButton("Previous");
		btnNext = new JButton("Next");
		btnDoSomething = new JButton("Segment");
		btnSegmentAll = new JButton("Segment All");
		btnSegmentAll.setEnabled(false);

		btnSaveSettings = new JButton("Save Settings");
		btnLoadSettings = new JButton("Load Settings");

		btnLppEdit = new JButton("Edit");
		// btnLppEdit.setEnabled(false);
		btnLppCalcAll = new JButton("Calc Files");
		// btnLppCalcAll.setEnabled(false);
		btnLppCalc = new JButton("Calc");
		// btnLppCalc.setEnabled(false);
		btnLPPAuto = new JButton("Auto");
		// btnLPPAuto.setEnabled(false);

		cbSegOnSwitch = new JCheckBox("On Switch");
		cbFullAuto = new JCheckBox("Full Auto");
	}

	public int getVerticalResolution() {
		return verticalResolution;
	}

	public void setVerticalResolution(int verticalResolution) {
		this.verticalResolution = verticalResolution;
	}

	public GuiManager getGuiManager() {
		return guiManager;
	}

	public void setGuiManager(GuiManager guiManager) {
		this.guiManager = guiManager;
	}

	public DataManager getDataManager() {
		return dataManager;
	}

	public void setDataManager(DataManager dataManager) {
		this.dataManager = dataManager;
	}

	public PnlRegionParameters getPnlRegionParameters() {
		return pnlRegionParameters;
	}

	public void setPnlRegionParameters(PnlRegionParameters pnlRegionParameters) {
		this.pnlRegionParameters = pnlRegionParameters;
	}

	public PnlProcessParameters getPnlProcessParameters() {
		return pnlProcessParameters;
	}

	public void setPnlProcessParameters(PnlProcessParameters pnlProcessParameters) {
		this.pnlProcessParameters = pnlProcessParameters;
	}

	public PnlPages getPnlPages() {
		return pnlPages;
	}

	public void setPnlPages(PnlPages pnlPages) {
		this.pnlPages = pnlPages;
	}

	public JRadioButton getRbOriginal() {
		return rbOriginal;
	}

	public void setRbOriginal(JRadioButton rbOriginal) {
		this.rbOriginal = rbOriginal;
	}

	public JRadioButton getRbBinary() {
		return rbBinary;
	}

	public void setRbBinary(JRadioButton rbBinary) {
		this.rbBinary = rbBinary;
	}

	public JRadioButton getRbResult() {
		return rbResult;
	}

	public void setRbResult(JRadioButton rbResult) {
		this.rbResult = rbResult;
	}

	public JRadioButton getRbPostProc() {
		return rbPostProc;
	}

	public void setRbPostProc(JRadioButton rbPostProc) {
		this.rbPostProc = rbPostProc;
	}

	public BackgroundPanel getPnlBackground() {
		return pnlBackground;
	}

	public void setPnlBackground(BackgroundPanel pnlBackground) {
		this.pnlBackground = pnlBackground;
	}

	public ResultsPanel getPnlResults() {
		return pnlResults;
	}

	public void setPnlResults(ResultsPanel pnlResults) {
		this.pnlResults = pnlResults;
	}

	public PnlPostProcessing getPnlPostProcessing() {
		return pnlPostProcessing;
	}

	public void setPnlPostProcessing(PnlPostProcessing pnlPostProcessing) {
		this.pnlPostProcessing = pnlPostProcessing;
	}

	public JCheckBox getCbSegOnSwitch() {
		return cbSegOnSwitch;
	}

	public void setCbSegOnSwitch(JCheckBox cbSegOnSwitch) {
		this.cbSegOnSwitch = cbSegOnSwitch;
	}

	public JCheckBox getCbFullAuto() {
		return cbFullAuto;
	}

	public void setCbFullAuto(JCheckBox cbFullAuto) {
		this.cbFullAuto = cbFullAuto;
	}

	public JButton getBtnSegmentAll() {
		return btnSegmentAll;
	}

	public void setBtnSegmentAll(JButton btnSegmentAll) {
		this.btnSegmentAll = btnSegmentAll;
	}
}