package gui.drawing;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import org.opencv.core.Mat;

import positions.Position;
import positions.PositionManager;
import regions.RegionManager;
import segmentation.ImageProcessor;

@SuppressWarnings("serial")
public class BackgroundPanel extends JPanel {

	private BackgroundPanel pnl;

	private Image image;
	private Mat matImage;
	private Painter painter;

	private RegionManager regionManager;
	private PositionManager positionManager;

	public BackgroundPanel(Mat image, RegionManager regionManager) {
		setRegionManager(regionManager);
		setPositionManager(regionManager.getPositionManager());

		setPainter(new Painter(this));

		updateImage(image, true);
		setOpaque(false);

		addListeners();
		setPnl(this);
	}

	public void addListeners() {
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				painter.handleMousePressed(e);
				repaint();
			}

			public void mouseReleased(MouseEvent e) {
				painter.handleMouseReleased();
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				painter.handleMouseDragged(e);
				repaint();
			}

			public void mouseMoved(MouseEvent e) {
				painter.handleMouseMoved(e);
				repaint();
			}
		});
	}

	public void repaintAll() {
		repaint();
	}

	public void paint(Graphics g) {
		super.paint(g);
		painter.paint(g);
	}

	public void updateImage(Mat image, boolean removeFixed) {
		setMatImage(image);

		BufferedImage inputBI = ImageProcessor.matToBufferedImage(image);
		setPreferredSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));
		setSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));
		this.image = inputBI;

		regionManager.update(image, removeFixed);
		painter.setArea(new Rectangle(0, 0, inputBI.getWidth(null), inputBI.getHeight(null)));
		painter.setDrawingLines(false);
		
		repaint();
	}

	public void checkAndInitActiveRect(MouseEvent e) {
		Position position = positionManager.identifyActiveRect(e);

		if (position != null) {
			positionManager.manageActivePositions(position);
			painter.setActivePosition(position);
		} else {
			positionManager.manageActivePositions(position);
			painter.setActivePosition(position);
		}

		repaint();
	}

	public BackgroundPanel getPnl() {
		return pnl;
	}

	public void setPnl(BackgroundPanel pnl) {
		this.pnl = pnl;
	}

	public Image getImage() {
		return image;
	}

	public void setImage(Image image) {
		this.image = image;
	}

	public Mat getMatImage() {
		return matImage;
	}

	public void setMatImage(Mat matImage) {
		this.matImage = matImage;
	}

	public RegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(RegionManager regionManager) {
		this.regionManager = regionManager;
	}

	public PositionManager getPositionManager() {
		return positionManager;
	}

	public void setPositionManager(PositionManager positionManager) {
		this.positionManager = positionManager;
	}

	public Painter getPainter() {
		return painter;
	}

	public void setPainter(Painter painter) {
		this.painter = painter;
	}
}