package gui.drawing;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;

import geometry.PointList;
import gui.GuiManager;
import net.miginfocom.swing.MigLayout;
import positions.Position;
import regions.RegionManager;
import segmentation.result.SegmentationResult;
import helper.TypeConverter;

@SuppressWarnings("serial")
public class DiaDrawnRect extends JDialog {

	private DiaDrawnRect popUp;

	private JButton btnMerge;
	private JButton btnSetRoi;
	private JButton btnSetPermaRoi;
	private JButton btnZoom;
	private JComboBox<String> cobTypes;

	public DiaDrawnRect(Point point, Position position, Painter painter) {
		super();

		setTitle("Set Type");

		createWidgets();
		addWidgets();
		addListener(position, painter);

		setPopUp(this);

		setModal(true);
		setLocation(point);
		setResizable(false);
		pack();
		setVisible(true);
	}

	private void addListener(final Position position, final Painter painter) {
		final RegionManager regionManager = painter.getRegionManager();
		final GuiManager guiManager = regionManager.getGui().getGuiManager();

		btnMerge.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Position roi = guiManager.getGui().getPnlBackground().getPainter().getActivePosition();

				if (roi != null) {
					SegmentationResult segResult = guiManager.getCurrentPage().getSegmentationResult();

					if (segResult != null) {
						segResult.mergeRegions(roi, guiManager.getGui());
						guiManager.getCurrentPage().visualizeSegmentation();

						if (guiManager.getGui().getRbPostProc().isSelected()) {
							guiManager.getGui().getPnlResults().repaintAll();
						} else {
							guiManager.getGui().getRbPostProc().doClick();
						}
						popUp.dispose();
					}
				}
			}
		});

		btnSetRoi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Position roi = painter.getActivePosition();

				if (roi != null) {
					guiManager.getRegionManager().setRoI(roi, painter, false);
				}

				guiManager.getRegionManager().getTempRegion().setPositions(new ArrayList<Position>());

				painter.getPanel().repaintAll();
				popUp.dispose();
			}
		});

		btnSetPermaRoi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Position roi = painter.getActivePosition();

				if (roi != null) {
					guiManager.getRegionManager().setRoI(roi, painter, true);
				}

				guiManager.getRegionManager().getTempRegion().setPositions(new ArrayList<Position>());

				painter.getPanel().repaintAll();
				popUp.dispose();
			}
		});

		cobTypes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ArrayList<java.awt.Point> points = new ArrayList<java.awt.Point>();
				points.add(new Point((int) position.getDrawingRect().getX(), (int) position.getDrawingRect().getY()));
				points.add(
						new Point((int) position.getDrawingRect().getX() + (int) position.getDrawingRect().getWidth(),
								(int) position.getDrawingRect().getY()));
				points.add(
						new Point((int) position.getDrawingRect().getX() + (int) position.getDrawingRect().getWidth(),
								(int) position.getDrawingRect().getY() + (int) position.getDrawingRect().getHeight()));
				points.add(new Point((int) position.getDrawingRect().getX(),
						(int) position.getDrawingRect().getY() + (int) position.getDrawingRect().getHeight()));

				PointList linePoints = new PointList(points);
				linePoints.setClosed(true);
				painter.getPointListManager().getPointLists().add(linePoints);
				linePoints.setType(TypeConverter.stringToType(cobTypes.getSelectedItem().toString()));

				regionManager.getPositionManager().deletePosition(position);

				painter.getPanel().repaintAll();
				popUp.dispose();
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout();
		setLayout(layout);

		add(cobTypes, "sg 1, wrap");
		add(btnMerge, "sg 1, wrap");
		// add(btnAddPos, "sg 1, wrap");
		add(btnSetRoi, "sg 1, wrap");
		add(btnSetPermaRoi, "sg 1, wrap");
		add(btnZoom, "sg 1, wrap");
	}

	private void createWidgets() {
		setContentPane(new JPanel());

		btnMerge = new JButton("Merge");
		new JButton("Add Position");
		btnSetRoi = new JButton("Set RoI");
		btnSetPermaRoi = new JButton("Set Perma RoI");
		btnZoom = new JButton("Zoom");

		String[] regionTypesString = RegionManager.getRegionNames();

		cobTypes = new JComboBox<String>(regionTypesString) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
	}

	public DiaDrawnRect getPopUp() {
		return popUp;
	}

	public void setPopUp(DiaDrawnRect popUp) {
		this.popUp = popUp;
	}
}