package gui.initParameters;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import net.miginfocom.swing.MigLayout;
import positions.PriorityPosition;
import regions.Region;
import regions.RegionManager;
import regions.colors.RegionColor;
import regions.colors.RegionColors;
import regions.type.RegionType;
import segmentation.parameters.DEFAULT_Parameters;

@SuppressWarnings("serial")
public class DiaNewRegion extends JDialog {

	private RegionManager regionManager;
	private ArrayList<Region> regions;

	private JLabel lblType;
	private JLabel lblColor;
	private JLabel lblMinSize;
	private JLabel lblMaxOcc;
	private JLabel lblPriorityPosition;

	private JPanel panel;

	private JButton btnCreate;

	private JRadioButton rbMaxOccOne;
	private JRadioButton rbMaxOccUnbounded;

	private JSpinner spinMinSize;

	private JComboBox<String> cobType;
	private JComboBox<String> cobColor;
	private JComboBox<String> cobPriorityPosition;

	public DiaNewRegion(RegionManager regionManager) {
		super(new JFrame(), "Create New Region");

		setRegionManager(regionManager);
		setRegions(regionManager.getRegions());

		createWidgets();
		addWidgets();
		addListener();

		pack();
		setLocationRelativeTo(null);
		setResizable(false);
		setModal(true);
		setVisible(true);
	}

	public void addListener() {
		btnCreate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				RegionType type = RegionType.valueOf(cobType.getSelectedItem().toString());
				int minSize = (int) spinMinSize.getValue();
				RegionColor color = RegionColors.getColorByName(cobColor.getSelectedItem().toString());
				int maxOccurences = -1;
				PriorityPosition priorityPosition = null;

				if (rbMaxOccOne.isSelected()) {
					maxOccurences = 1;
					priorityPosition = PriorityPosition.valueOf(cobPriorityPosition.getSelectedItem().toString());
				}

				Region newRegion = new Region(type, minSize, color, maxOccurences, priorityPosition, null);
				regionManager.addRegion(newRegion);
				newRegion.setVisible(true);

				dispose();
			}
		});

		rbMaxOccOne.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rbMaxOccOne.setSelected(true);
				rbMaxOccUnbounded.setSelected(false);
			}
		});

		rbMaxOccUnbounded.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rbMaxOccUnbounded.setSelected(true);
				rbMaxOccOne.setSelected(false);
			}
		});
	}

	public void addWidgets() {
		add(panel);

		MigLayout layout = new MigLayout();
		panel.setLayout(layout);

		panel.add(lblType);
		panel.add(cobType, "sg 1, wrap");
		panel.add(lblColor);
		panel.add(cobColor, "sg 1, wrap");
		panel.add(lblMinSize);
		panel.add(spinMinSize, "sg 1, wrap");
		panel.add(lblMaxOcc);
		panel.add(rbMaxOccOne, "split");
		panel.add(rbMaxOccUnbounded, "split, wrap");
		panel.add(lblPriorityPosition);
		panel.add(cobPriorityPosition, "sg 1, wrap");
		panel.add(btnCreate, "split, span, growx, pushx, wrap");
	}

	public void createWidgets() {
		panel = new JPanel();

		lblType = new JLabel("Type:");
		lblColor = new JLabel("Color:");
		lblMinSize = new JLabel("Min Size:");
		lblMaxOcc = new JLabel("Max Occ:");
		lblPriorityPosition = new JLabel("Priority:");

		btnCreate = new JButton("Create Region");

		rbMaxOccOne = new JRadioButton("1");
		rbMaxOccOne.setSelected(false);

		rbMaxOccUnbounded = new JRadioButton("Unbounded");
		rbMaxOccUnbounded.setSelected(true);

		spinMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getParagraphMinSizeDefault(),
				DEFAULT_Parameters.getParagraphMinSizeMin(), DEFAULT_Parameters.getImageMinSizeMax(),
				DEFAULT_Parameters.getParagraphMinSizeStepsize()));

		// TODO: auslagern! furchtbar!
		RegionType[] regionTypes = RegionType.values();
		String[] regionTypesString = new String[regionTypes.length - regions.size()];
		// System.out.println(regionTypesString.length);
		int cnt = 0;

		for (int i = 0; i < regionTypes.length; i++) {
			boolean contains = false;

			for (Region region : regions) {
				if (region.getPageXmlIdentifier().equals(regionTypes[i].toString())) {
					contains = true;
				}
			}

			if (!contains) {
				regionTypesString[cnt++] = regionTypes[i].toString();
			}
		}

		cobType = new JComboBox<String>(regionTypesString) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};

		cobColor = new JComboBox<String>(RegionColors.getUnusedColorNames(regions)) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};

		String[] priorityPositionsStrings = { "top", "bottom", "left", "right" };

		cobPriorityPosition = new JComboBox<String>(priorityPositionsStrings) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
	}

	public RegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(RegionManager regionManager) {
		this.regionManager = regionManager;
	}

	public ArrayList<Region> getRegions() {
		return regions;
	}

	public void setRegions(ArrayList<Region> regions) {
		this.regions = regions;
	}
}