package gui.initParameters;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultFormatter;

import gui.GuiManager;
import net.miginfocom.swing.MigLayout;
import segmentation.parameters.DEFAULT_Parameters;

@SuppressWarnings("serial")
public class PnlProcessParameters extends JPanel {

	private String layoutDebug;

	private JLabel lblImageSize;
	private JLabel lblBinaryThresh;
	private JLabel lblImageDilationX;
	private JLabel lblImageDilationY;
	private JLabel lblTextDilationX;
	private JLabel lblTextDilationY;
	private JLabel lblImageSegMode;

	private JSpinner spinImageSize;
	private JSpinner spinBinaryThresh;
	private JSpinner spinImageDilationX;
	private JSpinner spinImageDilationY;
	private JSpinner spinTextDilationX;
	private JSpinner spinTextDilationY;
	
	private JComboBox<String> cobImageSegMode;
	private JCheckBox cbCombine;

	public PnlProcessParameters(GuiManager guiManager, String layoutDebug) {
		super();

		setLayoutDebug(layoutDebug);

		createWidgets();
		addWidgets();
		addListener(guiManager);
	}

	private void addListener(final GuiManager guiManager) {
		cobImageSegMode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				String modeString = cobImageSegMode.getSelectedItem().toString();
				
				if(modeString.equals("Rotated Rect") || modeString.equals("Straight Rect")) {
					cbCombine.setSelected(true);
					cbCombine.setEnabled(true);
				} else {
					cbCombine.setSelected(false);
					cbCombine.setEnabled(false);
				}
			}
		});
		
		spinBinaryThresh.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				guiManager.calcAndShowBinary();
			}
		});
		
		spinImageDilationX.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if(Integer.parseInt(spinImageDilationX.getValue().toString()) == 0) {
					spinImageDilationY.setValue(0);
				} else if(Integer.parseInt(spinImageDilationY.getValue().toString()) == 0) {
					spinImageDilationY.setValue(spinImageDilationX.getValue());
				}
			}
		});
		
		spinImageDilationY.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if(Integer.parseInt(spinImageDilationY.getValue().toString()) == 0) {
					spinImageDilationX.setValue(0);
				} else if(Integer.parseInt(spinImageDilationX.getValue().toString()) == 0) {
					spinImageDilationX.setValue(spinImageDilationY.getValue());
				}
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(layoutDebug);
		setLayout(layout);

//		add(lblImageSize, "sg 1");
//		add(spinImageSize, "sg 2, wrap");
		add(lblBinaryThresh, "sg 1");
		add(spinBinaryThresh, "sg 2, wrap");
		
		add(lblTextDilationX, "sg 1");
		add(spinTextDilationX, "sg 2");
		add(lblImageDilationX, "sg 1");
		add(spinImageDilationX, "sg 2, wrap");
		
		add(lblTextDilationY, "sg 1");
		add(spinTextDilationY, "sg 2");
		add(lblImageDilationY, "sg 1");
		add(spinImageDilationY, "sg 2, wrap");
		
		add(lblImageSegMode, "split, span, pushx, growx");
		add(cobImageSegMode, "split, span, pushx, growx");
		add(cbCombine, "split, span, pushx, growx");
	}

	private void createWidgets() {
		setBorder(BorderFactory.createTitledBorder("Processing Parameters"));

		lblImageSize = new JLabel("Vertical Resolution");
		lblBinaryThresh = new JLabel("Binary Thresh");
		lblImageDilationX = new JLabel("Image Dilation X");
		lblImageDilationY = new JLabel("Image Dilation Y");
		lblTextDilationX = new JLabel("Text Dilation X");
		lblTextDilationY = new JLabel("Text Dilation Y");

		spinImageSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageHeightDefault(),
				DEFAULT_Parameters.getImageHeightMin(), DEFAULT_Parameters.getImageHeightMax(),
				DEFAULT_Parameters.getImageHeightStepsize()));
		spinBinaryThresh = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getBinaryThreshDefault(),
				DEFAULT_Parameters.getBinaryThreshMin(), DEFAULT_Parameters.getBinaryThreshMax(),
				DEFAULT_Parameters.getBinaryThreshStepsize()));
		spinImageDilationX = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageRemovalDilationXDefault(),
				DEFAULT_Parameters.getImageRemovalDilationXMin(), DEFAULT_Parameters.getImageRemovalDilationXMax(),
				DEFAULT_Parameters.getImageRemovalDilationXStepsize()));
		spinImageDilationY = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageRemovalDilationYDefault(),
				DEFAULT_Parameters.getImageRemovalDilationYMin(), DEFAULT_Parameters.getImageRemovalDilationYMax(),
				DEFAULT_Parameters.getImageRemovalDilationYStepsize()));
		spinTextDilationX = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getTextRemovalDilationXDefault(),
				DEFAULT_Parameters.getTextRemovalDilationXMin(), DEFAULT_Parameters.getTextRemovalDilationXMax(),
				DEFAULT_Parameters.getTextRemovalDilationXStepsize()));
		spinTextDilationY = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getTextRemovalDilationYDefault(),
				DEFAULT_Parameters.getTextRemovalDilationYMin(), DEFAULT_Parameters.getTextRemovalDilationYMax(),
				DEFAULT_Parameters.getTextRemovalDilationYStepsize()));

		JComponent comp = spinBinaryThresh.getEditor();
		JFormattedTextField field = (JFormattedTextField) comp.getComponent(0);
		DefaultFormatter formatter = (DefaultFormatter) field.getFormatter();
		formatter.setCommitsOnValidEdit(true);
		
		lblImageSegMode = new JLabel("Image Seg Mode");
		
		cbCombine = new JCheckBox("Combine");
		cbCombine.setSelected(true);
		
		String[] imageSegModes = new String[4];
		
		imageSegModes[0] = "Contour Only";
		imageSegModes[1] = "Straight Rect";
		imageSegModes[2] = "Rotated Rect";
		imageSegModes[3] = "None";

		cobImageSegMode = new JComboBox<String>(imageSegModes) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
		
		cobImageSegMode.setSelectedIndex(2);
	}

	public String getLayoutDebug() {
		return layoutDebug;
	}

	public void setLayoutDebug(String layoutDebug) {
		this.layoutDebug = layoutDebug;
	}

	public JLabel getLblImageSize() {
		return lblImageSize;
	}

	public JLabel getLblBinaryThresh() {
		return lblBinaryThresh;
	}

	public JLabel getLblImageDilationX() {
		return lblImageDilationX;
	}

	public JLabel getLblImageDilationY() {
		return lblImageDilationY;
	}

	public JLabel getLblTextDilationX() {
		return lblTextDilationX;
	}

	public JLabel getLblTextDilationY() {
		return lblTextDilationY;
	}

	public JSpinner getSpinImageSize() {
		return spinImageSize;
	}

	public JSpinner getSpinBinaryThresh() {
		return spinBinaryThresh;
	}

	public JSpinner getSpinImageDilationX() {
		return spinImageDilationX;
	}

	public JSpinner getSpinImageDilationY() {
		return spinImageDilationY;
	}

	public JSpinner getSpinTextDilationX() {
		return spinTextDilationX;
	}

	public JSpinner getSpinTextDilationY() {
		return spinTextDilationY;
	}

	public JComboBox<String> getCobImageSegMode() {
		return cobImageSegMode;
	}

	public void setCobImageSegMode(JComboBox<String> cobImageSegMode) {
		this.cobImageSegMode = cobImageSegMode;
	}

	public JCheckBox getCbCombine() {
		return cbCombine;
	}

	public void setCbCombine(JCheckBox cbCombine) {
		this.cbCombine = cbCombine;
	}
}