package gui.initParameters;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import gui.Gui;
import gui.postProcessing.PnlPostProcessing;
import net.miginfocom.swing.MigLayout;
import positions.Position;
import regions.Region;
import regions.RegionManager;
import regions.type.RegionType;
import segmentation.parameters.DEFAULT_Parameters;

@SuppressWarnings("serial")
public class PnlRegionParameters extends JPanel {

	private String layoutDebug;

	private JPanel pnlFixedRegions;

	private JLabel lblMinSize;

	private JCheckBox cbPermanentRegion;
	private JSpinner spinMinSize;

	private JButton btnHide;
	private JButton btnSet;
	private JButton btnNew;
	private JButton btnDelete;
	private JButton btnUpdate;

	private JComboBox<String> cobRegionPositions;
	private JComboBox<String> cobSegmentTypes;

	private RegionManager regionManager;

	public PnlRegionParameters(Gui gui, String layoutDebug) {
		super();

		setRegionManager(gui.getGuiManager().getRegionManager());
		setLayoutDebug(layoutDebug);

		createWidgets();
		addWidgets();
		addListener(gui);
	}

	public void deleteAllRegions(Gui gui) {
		for (int i = cobRegionPositions.getItemCount() - 1; i > -1; i--) {
			Region tempRegion = getSelectedRegion(gui);
			regionManager.getRegions().remove(tempRegion);
			cobRegionPositions.removeItemAt(i);
		}

		PnlPostProcessing.updateItems(gui.getGuiManager().getRegionManager());
		repaint();
	}

	// ToDo
	private Region getSelectedRegion(Gui gui) {
		if (cobRegionPositions.getSelectedItem() != null) {
			String regionType = cobRegionPositions.getSelectedItem().toString();
			regionType = regionType.toLowerCase();
			regionType = regionType.replace("-", "_");
			Region region = regionManager.getRegion(regionType);

			return region;
		} else {
			return null;
		}
	}

	private void addListener(final Gui gui) {
		btnHide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for (Region region : regionManager.getRegions()) {
					region.setVisible(false);
				}
			}
		});

		btnSet.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Position activePosition = regionManager.getPositionManager().getActivePosition();

				if (activePosition != null) {
					activePosition.setFixed(true);
					activePosition.setPermanent(cbPermanentRegion.isSelected());

					if (cobSegmentTypes.getSelectedItem().toString().equals("image")) {
						regionManager.getPositionManager().processFixed(activePosition, RegionType.image);
						cobRegionPositions.setSelectedItem("image");
					} else {
						regionManager.getPositionManager().processFixed(activePosition, RegionType.ignore);
						cobRegionPositions.setSelectedItem("ignore");
					}
				}
			}
		});

		btnUpdate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Region activeRegion = getSelectedRegion(gui);
				activeRegion.setMinSize((int) spinMinSize.getValue());

				for (Position position : regionManager.getTempRegion().getPositions()) {
					activeRegion.addPosition(position);
				}

				activeRegion.setVisible(true);
				regionManager.getTempRegion().setPositions(new ArrayList<Position>());
			}
		});

		btnDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Region region = getSelectedRegion(gui);
				regionManager.getRegions().remove(region);
				cobRegionPositions.removeItemAt(cobRegionPositions.getSelectedIndex());
				PnlPostProcessing.updateItems(gui.getGuiManager().getRegionManager());
				repaint();
			}
		});

		btnNew.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new DiaNewRegion(regionManager);
			}
		});

		cobRegionPositions.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Region region = getSelectedRegion(gui);
				if (region != null) {
					regionManager.changeActiveRegion(region.getPageXmlIdentifier());
					spinMinSize.setValue(region.getMinSize());

					if (region.getType().equals(RegionType.image) || region.getType().equals(RegionType.ignore)
							|| region.getType().equals(RegionType.paragraph)) {
						btnDelete.setEnabled(false);
					} else {
						btnDelete.setEnabled(true);
					}
				}
				gui.getPnlBackground().repaint();
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(layoutDebug);
		setLayout(layout);
		add(cobRegionPositions, "growx, wrap");
		add(lblMinSize, "split");
		add(spinMinSize, "split, pushx, growx, wrap");
		add(btnNew, "split, pushx, growx");
		add(btnUpdate, "split, pushx, growx");
		add(btnDelete, "split, pushx, growx, wrap");

		add(pnlFixedRegions, "growx, wrap");

		add(btnHide, "pushx, growx");

		MigLayout regionLayout = new MigLayout(layoutDebug);
		pnlFixedRegions.setLayout(regionLayout);
		pnlFixedRegions.add(btnSet, "growx, pushx");
		pnlFixedRegions.add(cobSegmentTypes, "growx, pushx");
		pnlFixedRegions.add(cbPermanentRegion, "growx, pushx, wrap");
	}

	private void createWidgets() {
		setBorder(BorderFactory.createTitledBorder("Regions"));

		lblMinSize = new JLabel("Minimum Region Size: ");
		spinMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageMinSizeDefault(),
				DEFAULT_Parameters.getImageMinSizeMin(), DEFAULT_Parameters.getImageMinSizeMax(),
				DEFAULT_Parameters.getImageMinSizeStepsize()));

		pnlFixedRegions = new JPanel();
		pnlFixedRegions.setBorder(BorderFactory.createTitledBorder("Fixed Position"));

		cbPermanentRegion = new JCheckBox("Permanent");
		new JCheckBox("All Fixed");

		btnHide = new JButton("Hide All Positions");
		btnSet = new JButton("Set");
		btnNew = new JButton("New");
		btnUpdate = new JButton("Update");
		btnDelete = new JButton("Delete");
		btnDelete.setEnabled(false);

		@SuppressWarnings("static-access")
		String[] regionTypes = regionManager.getRegionNames();

		cobRegionPositions = new JComboBox<String>(regionTypes) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};

		String[] fileFormats = { "image", "ignore" };

		cobSegmentTypes = new JComboBox<String>(fileFormats) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
	}

	public void refreshAddedRegion(Region region) {
		cobRegionPositions.addItem(region.getPageXmlIdentifier());
		cobRegionPositions.setSelectedItem(region.getPageXmlIdentifier());
		repaint();
	}

	public String getLayoutDebug() {
		return layoutDebug;
	}

	public void setLayoutDebug(String layoutDebug) {
		this.layoutDebug = layoutDebug;
	}

	public RegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(RegionManager regionManager) {
		this.regionManager = regionManager;
	}
}