package gui.postProcessing;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import gui.Gui;
import gui.readingOrder.ReadingOrderGui;
import net.miginfocom.swing.MigLayout;
import positions.Position;
import regions.Region;
import regions.RegionManager;
import regions.type.RegionType;
import segmentation.result.SegmentationResult;

@SuppressWarnings("serial")
public class PnlPostProcessing extends JPanel {

	private Gui gui;
	private ResultsPanel pnlResults;

	private JButton btnMerge;
	private JButton btnReadingOrder;

	private static JComboBox<String> cobTypes;

	public PnlPostProcessing(Gui gui) {
		super();

		setGui(gui);

		createWidgets();
		addWidgets();
		addListener();
	}

	private void addListener() {
		cobTypes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				setPnlResults(gui.getPnlResults());

				if (pnlResults != null && pnlResults.getPainter() != null
						&& pnlResults.getPainter().getActivePosition() != null) {
					if (pnlResults.getPainter().getActivePosition() != null) {
						String regionName = cobTypes.getSelectedItem().toString();
						Region region = gui.getGuiManager().getRegionManager().getRegion(regionName);

						if (region == null) {
							return;
						}
						RegionType type = region.getType();

						pnlResults.getPainter().getSegmentationResult().changeRegionType(
								pnlResults.getPainter().getActivePosition(), type,
								gui.getGuiManager().getRegionManager());

						pnlResults.getPainter().setActivePosition(null);
						pnlResults.getPainter().setColorActivePosition(null);
						pnlResults.getPainter().calcImage();
						pnlResults.repaint();
					}
				}
			}
		});

		btnMerge.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Position roi = gui.getPnlBackground().getPainter().getActivePosition();

				if (roi != null) {
					SegmentationResult segResult = gui.getGuiManager().getCurrentPage().getSegmentationResult();

					if (segResult != null) {
						segResult.mergeRegions(roi, gui);
						gui.getGuiManager().getCurrentPage().visualizeSegmentation();

						if (gui.getRbPostProc().isSelected()) {
							gui.getPnlResults().repaintAll();
						} else {
							gui.getRbPostProc().doClick();
						}
					}
				}
			}
		});

		btnReadingOrder.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (gui.getGuiManager().getCurrentPage().getSegmentationResult() != null) {
					ReadingOrderGui readingOrderGui = new ReadingOrderGui(gui.getGuiManager().getCurrentPage());
					readingOrderGui.setVisible(true);
				}
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout();
		setLayout(layout);

		add(btnReadingOrder, "sg 1, pushx, growx");
		add(cobTypes, "sg 1, pushx, growx");
	}

	private void createWidgets() {
		setBorder(BorderFactory.createTitledBorder("Post Processing"));
		btnMerge = new JButton("Merge");
		btnReadingOrder = new JButton("Reading Order");

		ArrayList<Region> regions = gui.getGuiManager().getRegionManager().getRegions();
		String[] regionTypesString = new String[regions.size() - 1];

		for (int i = 0; i < regions.size(); i++) {
			if (!regions.get(i).getPageXmlIdentifier().equals("ignore")) {
				regionTypesString[i] = regions.get(i).getPageXmlIdentifier();
			}
		}

		cobTypes = new JComboBox<String>(regionTypesString) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
	}

	public static void updateItems(RegionManager regionManager) {
		if (cobTypes != null) {
			cobTypes.removeAllItems();

			for (Region region : regionManager.getRegions()) {
				cobTypes.addItem(region.getPageXmlIdentifier());
			}
		}
	}

	public void refreshAddedRegion(Region region) {
		cobTypes.addItem(region.getPageXmlIdentifier());
	}

	public ResultsPanel getPnlResults() {
		return pnlResults;
	}

	public void setPnlResults(ResultsPanel pnlResults) {
		this.pnlResults = pnlResults;
	}

	public Gui getGui() {
		return gui;
	}

	public void setGui(Gui gui) {
		this.gui = gui;
	}
}