package importExport;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import positions.Position;
import regions.Region;
import segmentation.parameters.Parameters;

public class SettingsWriter {

	public static void saveSettings(String outputPath, Parameters parameters) {
		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document document = docBuilder.newDocument();

			Element rootElement = document.createElement("settings");
			document.appendChild(rootElement);

			Element parametersElement = document.createElement("parameters");

			parametersElement.setAttribute("verticalResolution", "" + parameters.getDesiredImageHeight());
			parametersElement.setAttribute("binaryThresh", "" + parameters.getBinaryThresh());
			parametersElement.setAttribute("imageDilationX", "" + parameters.getImageRemovalDilationX());
			parametersElement.setAttribute("imageDilationY", "" + parameters.getImageRemovalDilationY());
			parametersElement.setAttribute("textDilationX", "" + parameters.getTextDilationX());
			parametersElement.setAttribute("textDilationY", "" + parameters.getTextDilationY());

			Element regionsElement = document.createElement("regions");

			for (Region region : parameters.getRegionManager().getRegions()) {
				Element regionElement = document.createElement("region");

				regionElement.setAttribute("type", region.getType().toString());
				regionElement.setAttribute("color", region.getColor().getName());
				regionElement.setAttribute("minSize", "" + region.getMinSize());
				regionElement.setAttribute("maxOccurances", "" + region.getMaxOccurances());
				
				if(region.getMaxOccurances() == 1) {
					regionElement.setAttribute("priority", region.getPriorityPosition().toString());
				}

				for (Position position : region.getPositions()) {
					Element positionElement = document.createElement("position");

					positionElement.setAttribute("x1", "" + position.getTopLeftXPercentage());
					positionElement.setAttribute("y1", "" + position.getTopLeftYPercentage());
					positionElement.setAttribute("x2", "" + position.getBottomRightXPercentage());
					positionElement.setAttribute("y2", "" + position.getBottomRightYPercentage());
					positionElement.setAttribute("fixed", "" + position.isFixed());
					positionElement.setAttribute("permanent", "" + position.isPermanent());

					regionElement.appendChild(positionElement);
				}

				regionsElement.appendChild(regionElement);
			}
			
			rootElement.appendChild(parametersElement);
			rootElement.appendChild(regionsElement);

			// write content into xml file
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(document);

			if(!outputPath.endsWith(File.separator)) {
				outputPath += File.separator;
			}
			
			outputPath += "Settings.xml";
			
			StreamResult result = new StreamResult(new File(outputPath));
			transformer.transform(source, result);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}