package linePP;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;

import dataManagement.Page;
import linePP.data.LinePage;
import linePP.data.Segment;
import linePP.drawing.BackgroundPanel;
import linePP.drawing.LineRegionManager;
import net.miginfocom.swing.MigLayout;
import segmentation.result.ResultRegion;
import segmentation.result.SegmentationResult;

@SuppressWarnings("serial")
public class GuiLinePP extends JDialog {

	private int verticalResolution;
	// private Page page;
	private BackgroundPanel pnlBackground;

	private JPanel pnlButtons;
	private JButton btnSave;
	private JButton btnReset;

	public GuiLinePP(LinePage linePage, Page page, String outputFolder, int verticalResolution) {
		super();

		setVerticalResolution(verticalResolution);

		createWidgets(linePage);
		addWidgets();
		addListener(linePage, page);

		pack();
		setLocationRelativeTo(null);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setModal(true);
		setResizable(false);
	}

	public void addListener(final LinePage linePage, final Page page) {
		btnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ArrayList<ResultRegion> regions = new ArrayList<ResultRegion>();

				for (Segment segment : linePage.getSegments()) {
					segment.resizeAll((double) page.getOriginal().height() / verticalResolution);
					ResultRegion region = new ResultRegion(segment.getType(), verticalResolution,
							segment.getResizedRegion());
					regions.add(region);
				}

				SegmentationResult segResult = new SegmentationResult(regions, verticalResolution,
						linePage.getScaleFactor());

				page.setSegmentationResult(segResult);
				page.setScaleFactor(linePage.getScaleFactor());
				dispose();
			}
		});

		btnReset.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {

			}
		});
	}

	public void addWidgets() {
		MigLayout layout = new MigLayout();
		setLayout(layout);
		add(pnlBackground, "wrap");
		add(pnlButtons, "center");

		MigLayout layoutBtn = new MigLayout();
		pnlButtons.setLayout(layoutBtn);
		pnlButtons.add(btnSave, "sg 1");
		pnlButtons.add(btnReset, "sg 1");
	}

	public void createWidgets(LinePage linePage) {
		setContentPane(new JPanel());

		pnlBackground = new BackgroundPanel(linePage, new LineRegionManager(linePage));
		pnlButtons = new JPanel();

		btnSave = new JButton("Save");
		btnReset = new JButton("Reset");
	}

	public int getVerticalResolution() {
		return verticalResolution;
	}

	public void setVerticalResolution(int verticalResolution) {
		this.verticalResolution = verticalResolution;
	}
}