package linePP.auto;

import java.util.ArrayList;

import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;

import imageProcessing.Contour;
import imageProcessing.ImageProcessor;
import linePP.data.Line;
import linePP.data.LinePage;
import linePP.data.Segment;
import linePP.drawing.Cutter;
import regions.type.RegionType;

public class Merger {
	public static void merge(ArrayList<Line> toMerge, Segment segment, LinePage linePage, Line aboveLine,
			Line belowLine, RegionType type) {
		Mat binary = new Mat(linePage.getOriginal().size(), CvType.CV_8UC1, new Scalar(0));
		ArrayList<MatOfPoint> contours = new ArrayList<MatOfPoint>();

		for (Line line : toMerge) {
			contours.add(line.getLinePoints());
		}

		Imgproc.drawContours(binary, contours, -1, new Scalar(255), -1);

		// TODO: ray intersection
		Point lastCoG = ImageProcessor.calcCenterOfGravity(contours.get(0));

		for (int i = 1; i < toMerge.size(); i++) {
			Point currentCoG = ImageProcessor.calcCenterOfGravity(contours.get(i));

			Core.line(binary, lastCoG, currentCoG, new Scalar(255), 5);
			lastCoG = currentCoG;
		}

		contours = Contour.findContours(binary);

		if (aboveLine != null) {
			Cutter.performCut(linePage, segment, aboveLine, false);
		} else if (belowLine != null) {
			Cutter.performCut(linePage, segment, toMerge.get(toMerge.size() - 1), false);
		}

		if (type != null) {
			for (Segment newSegment : linePage.getSegments()) {
				if (newSegment.getLines().contains(toMerge.get(0))) {
					newSegment.setType(type);
				}
			}
		}

		for (Segment newSegment : linePage.getSegments()) {
			for (Line line : toMerge) {
				if (newSegment.getLines().contains(line)) {
					newSegment.setRegion(contours.get(0));

					break;
				}
			}
		}

		for (Line line : toMerge) {
			line.setLineType(LineType.NORMAL);
		}
	}

	public static boolean mergeLines(LinePage linePage) {
		for (Segment segment : linePage.getSegments()) {
			if (!segment.getType().equals(RegionType.heading)) {
				for (int i = 0; i < segment.getLines().size(); i++) {
					Line line = segment.getLines().get(i);

					if (!line.getLineType().equals(LineType.NORMAL)
							&& !line.getLineType().equals(LineType.HIGH_CANDIDATE)) {
						ArrayList<Line> toMerge = new ArrayList<Line>();
						toMerge.add(line);
						boolean keepSearching = true;
						int index = i + 1;

						while (keepSearching) {
							if (index < segment.getLines().size()) {
								Line nextLine = segment.getLines().get(index);

								if (nextLine.getLineType().equals(line.getLineType())) {
									toMerge.add(nextLine);
									index++;
								} else {
									keepSearching = false;
								}
							} else {
								keepSearching = false;
							}
						}

						if (toMerge.size() > 0) {
							Line aboveLine = null;
							Line belowLine = null;

							if (i > 0) {
								aboveLine = segment.getLines().get(i - 1);
							}

							if (i < segment.getLines().size() - 1) {
								belowLine = segment.getLines().get(i + 1);
							}

							merge(toMerge, segment, linePage, aboveLine, belowLine, null);
							linePage.update();

							return false;
						}
					}
				}
			}
		}

		return true;
	}
}