package linePP.calc;

import java.io.File;
import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.highgui.Highgui;
import org.opencv.imgproc.Imgproc;

import linePP.helper.ImageProcessor;
import linePP.pageXML.Export;
import linePP.pageXML.Import;

public class LinePP {

	private String outputFolder;

	public LinePP(String inputFolder) {
		initOutputFolder(inputFolder);
	}

	public void calcLines(dataManagement.Page resultPage) {
		Page page = new Page(resultPage);
		Mat image = Highgui.imread(resultPage.getImagePath());

		page.setMatImage(image);

		if (image.channels() > 1) {
			Imgproc.cvtColor(image, image, Imgproc.COLOR_BGR2GRAY);
		}

		LineSeparator lineSeparator = new LineSeparator(page, image, true);
		ArrayList<Segment> segments = lineSeparator.calcLines(true);

		Export.writePageXML(page, segments, outputFolder);
		
		for (Segment segment : segments) {
			segment.clean();
		}

		ImageProcessor.releaseAll(image);
	}

	public void calcLines(File file) {
		String xmlPath = file.getAbsolutePath();
		String imagePath = xmlPath.replace(File.separator + "SegmentXMLs", "");
		imagePath = imagePath.replace(".xml", ".tif");

		File imageFile = new File(imagePath);

		if (imageFile.exists()) {
			Page page = Import.readPageXML(xmlPath);
			Mat image = Highgui.imread(imagePath);
			page.setMatImage(image);

			if (image.channels() > 1) {
				Imgproc.cvtColor(image, image, Imgproc.COLOR_BGR2GRAY);
			}

			LineSeparator lineSeparator = new LineSeparator(page, image, true);
			ArrayList<Segment> segments = lineSeparator.calcLines(true);

			Export.writePageXML(page, segments, outputFolder);

			for (Segment segment : segments) {
				segment.clean();
			}

			ImageProcessor.releaseAll(image, page.getMatImage());

			System.gc();
		}
	}

	private void initOutputFolder(String inputFolder) {
		if (!inputFolder.endsWith(File.separator)) {
			inputFolder += File.separator;
		}

		outputFolder = inputFolder + "LineXMLs" + File.separator;
		new File(outputFolder).mkdirs();
	}

	public String getOutputFolder() {
		return outputFolder;
	}
}