package linePP.calc;

import java.util.ArrayList;

import org.opencv.core.MatOfPoint;

public class LinePostProc {

	private static final double CONTAIN_THRESH = 0.7;

	public static ArrayList<MatOfPoint> postProcessLines(ArrayList<MatOfPoint> lines) {
		ArrayList<PPRect> ppRects = calcPPRectsFromLines(lines);
		removeContainedLines(ppRects);

		return calcLinesFromPPRects(ppRects);
	}

	private static void removeContainedLines(ArrayList<PPRect> lines) {
		ArrayList<PPRect> removedRects = new ArrayList<PPRect>();

		for (int i = 0; i < lines.size() - 1; i++) {
			PPRect line0 = lines.get(i);
			PPRect line1 = lines.get(i + 1);

			if (line0.getRect().area() > line1.getRect().area()) {
				if (line0.containsEnoughRect(line1, CONTAIN_THRESH)) {
					if (!removedRects.contains(line0)) {
						removedRects.add(line1);
					}
				}
			} else {
				if (line1.containsEnoughRect(line0, CONTAIN_THRESH)) {
					if (!removedRects.contains(line1)) {
						removedRects.add(line0);
					}
				}
			}
		}

		lines.removeAll(removedRects);
	}

	private static ArrayList<MatOfPoint> calcLinesFromPPRects(ArrayList<PPRect> rects) {
		ArrayList<MatOfPoint> lines = new ArrayList<MatOfPoint>();

		for (PPRect rect : rects) {
			lines.add(rect.getPoints());
		}

		return lines;
	}

	private static ArrayList<PPRect> calcPPRectsFromLines(ArrayList<MatOfPoint> lines) {
		ArrayList<PPRect> rects = new ArrayList<PPRect>();

		for (MatOfPoint line : lines) {
			PPRect rect = new PPRect(line);
			rects.add(rect);
		}

		return rects;
	}
}