package linePP.calc;

import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.imgproc.Imgproc;

public class PPRect {

	private MatOfPoint points;
	private Point[] pointArray;
	private Rect rect;

	public PPRect(MatOfPoint points) {
		setPoints(points);
		setPointArray(points.toArray());
		setRect(Imgproc.boundingRect(points));
	}

	public boolean containsEnoughRect(PPRect toCheck, double reqAreaPerc) {
		if (this.containsPoint(toCheck.getPointArray())) {
			double intersectingArea = calcIntersectionArea(toCheck.getRect());
			double intersectPerc = intersectingArea / toCheck.rect.area();

			if (intersectPerc > reqAreaPerc) {
				return true;
			}
		}

		return false;
	}

	public double calcIntersectionArea(Rect toCheck) {
		double x_overlap = Math.max(0,
				Math.min(this.rect.x + this.rect.width, toCheck.x + toCheck.width) - Math.max(this.rect.x, toCheck.x));
		double y_overlap = Math.max(0, Math.min(this.rect.y + this.rect.height, toCheck.y + toCheck.height)
				- Math.max(this.rect.y, toCheck.y));

		double area = x_overlap * y_overlap;

		return area;
	}

	public boolean containsPoint(Point[] pointArray) {
		for (int i = 0; i < pointArray.length; i++) {
			if (this.rect.contains(pointArray[i])) {
				return true;
			}
		}

		return false;
	}

	public MatOfPoint getPoints() {
		return points;
	}

	public void setPoints(MatOfPoint points) {
		this.points = points;
	}

	public Point[] getPointArray() {
		return pointArray;
	}

	public void setPointArray(Point[] pointArray) {
		this.pointArray = pointArray;
	}

	public Rect getRect() {
		return rect;
	}

	public void setRect(Rect rect) {
		this.rect = rect;
	}
}