package linePP.data;

import java.io.File;
import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.Size;
import org.opencv.highgui.Highgui;
import org.opencv.imgproc.Imgproc;

import segmentation.result.SegmentationResult;

public class LinePage {

	private String imagePath;
	private String imageFilename;
	private Mat original;
	private Size imageSize;
	private ArrayList<Segment> segments;
	private SegmentationResult segResult;

	private Mat resized;
	double scaleFactor;

	public LinePage(String imagePath, ArrayList<Segment> segments) {
		setImagePath(imagePath);
		setSegments(segments);
//		System.out.println("bla" + imagePath + "bla");
		setImageFilename(imagePath.substring(imagePath.lastIndexOf(File.separator) + 1, imagePath.lastIndexOf(".")));
	}

	public void init(int desiredHeight) {
		Mat original = Highgui.imread(imagePath);
		setOriginal(original);
		double scaleFactor = (double) original.height() / desiredHeight;
		setScaleFactor(scaleFactor);

		Mat resized = new Mat();
		Imgproc.resize(original, resized, new Size(original.width() / scaleFactor, desiredHeight));
		setResized(resized);
	}

	public void update() {
		for (Segment segment : segments) {
			segment.resizeAll(scaleFactor);
		}
	}

	public void clean() {
		if (original != null) {
			original.release();
			setOriginal(null);
		}
		if (resized != null) {
			resized.release();
			setResized(null);
		}
	}

	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		this.imagePath = imagePath;
	}

	public String getImageFilename() {
		return imageFilename;
	}

	public void setImageFilename(String imageFilename) {
		this.imageFilename = imageFilename;
	}

	public Mat getOriginal() {
		return original;
	}

	public void setOriginal(Mat original) {
		this.original = original;
	}

	public ArrayList<Segment> getSegments() {
		return segments;
	}

	public void setSegments(ArrayList<Segment> segments) {
		this.segments = segments;
	}

	public Mat getResized() {
		return resized;
	}

	public void setResized(Mat resized) {
		this.resized = resized;
	}

	public double getScaleFactor() {
		return scaleFactor;
	}

	public void setScaleFactor(double scaleFactor) {
		this.scaleFactor = scaleFactor;
	}

	public Size getImageSize() {
		return imageSize;
	}

	public void setImageSize(Size imageSize) {
		this.imageSize = imageSize;
	}

	public SegmentationResult getSegResult() {
		return segResult;
	}

	public void setSegResult(SegmentationResult segResult) {
		this.segResult = segResult;
	}
}