package linePP.data;

import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.imgproc.Imgproc;

import geometry.StraightLine;
import regions.type.RegionType;

public class Segment {

	private MatOfPoint region;
	private RegionType type;
	private ArrayList<Line> lines;

	private MatOfPoint ResizedRegion;
	
	private Polygon polyRegion;
	private Marginalia marginalia;

	private boolean isActive;
	private boolean isLeft;
	private boolean isPruned;

	private StraightLine leftLine;

	public Segment(MatOfPoint region, RegionType type) {
		setRegion(region);
		setType(type);
		setLines(new ArrayList<Line>());
	}

	public void calcLeftLine(Mat image) {
		ArrayList<Point> pointList = new ArrayList<Point>();

		double top = Double.MAX_VALUE;
		double bottom = 0;

		for (Line line : lines) {
			Point point = line.getTopLeft();
			pointList.add(point);

			if (point.y < top) {
				top = point.y;
			}

			if (point.y > bottom) {
				bottom = point.y;
			}
		}

		Point[] points = pointList.toArray(new Point[pointList.size()]);
		Mat fittedLine = new Mat();
		Imgproc.fitLine(new MatOfPoint(points), fittedLine, Imgproc.CV_DIST_L1, 0, 0.01, 0.01);

		double vw = fittedLine.get(0, 0)[0];
		double vy = fittedLine.get(1, 0)[0];
		double x0 = fittedLine.get(2, 0)[0];
		double y0 = fittedLine.get(3, 0)[0];

		StraightLine straightLine = new StraightLine(x0, y0, vw, vy, top, bottom);
		setLeftLine(straightLine);
	}

	public Line identifyClickedLine(Point point) {
		for (Line line : lines) {
			if (Imgproc.pointPolygonTest(new MatOfPoint2f(line.getResizedLine().getLinePoints().toArray()), point,
					false) > 0) {
				return line;
			}
		}

		return null;
	}

	private MatOfPoint resize(MatOfPoint input, double scaleFactor) {
		Point points[] = input.toArray();
		Point resizedPoints[] = new Point[points.length];

		for (int i = 0; i < points.length; i++) {
			Point point = new Point(points[i].x / scaleFactor, points[i].y / scaleFactor);
			resizedPoints[i] = point;
		}

		return new MatOfPoint(resizedPoints);
	}

	public void resizeAll(double scaleFactor) {
		MatOfPoint resRegion = resize(region, scaleFactor);
		setResizedRegion(resRegion);
		setPolyRegion(new Polygon(resRegion));

		for (Line line : lines) {
			Line resizedLine = new Line(resize(line.getLinePoints(), scaleFactor), line, line.getType());
			resizedLine.setPolygon(new Polygon(resizedLine.getLinePoints()));
			line.setResizedLine(resizedLine);
		}
	}

	public void addLine(Line line) {
		lines.add(line);
	}

	public MatOfPoint getRegion() {
		return region;
	}

	public void setRegion(MatOfPoint region) {
		this.region = region;
	}

	public RegionType getType() {
		return type;
	}

	public void setType(RegionType type) {
		this.type = type;
	}

	public ArrayList<Line> getLines() {
		return lines;
	}

	public void setLines(ArrayList<Line> lines) {
		this.lines = lines;
	}

	public MatOfPoint getResizedRegion() {
		return ResizedRegion;
	}

	public void setResizedRegion(MatOfPoint resizedRegion) {
		ResizedRegion = resizedRegion;
	}

	public Polygon getPolyRegion() {
		return polyRegion;
	}

	public void setPolyRegion(Polygon polyRegion) {
		this.polyRegion = polyRegion;
	}

	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public Marginalia getMarginalia() {
		return marginalia;
	}

	public void setMarginalia(Marginalia marginalia) {
		this.marginalia = marginalia;
	}

	public boolean isLeft() {
		return isLeft;
	}

	public void setLeft(boolean isLeft) {
		this.isLeft = isLeft;
	}

	public StraightLine getLeftLine() {
		return leftLine;
	}

	public void setLeftLine(StraightLine leftLine) {
		this.leftLine = leftLine;
	}

	public boolean isPruned() {
		return isPruned;
	}

	public void setPruned(boolean isPruned) {
		this.isPruned = isPruned;
	}
}