package linePP.dataImport;

import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;

import linePP.data.Line;
import linePP.data.LinePage;
import linePP.data.Segment;

public class Import {

	private static Rect calcExtendedRect(MatOfPoint points, Size size) {
		Rect rect = Imgproc.boundingRect(points);
		Point topLeft = new Point(rect.tl().x - 0.2 * rect.width, rect.tl().y - 0.2 * rect.height);
		Point bottomRight = new Point(rect.br().x + 0.2 * rect.width, rect.br().y + 0.2 * rect.height);

		if (topLeft.x < 0)
			topLeft.x = 0;
		if (topLeft.y < 0)
			topLeft.y = 0;
		if (bottomRight.x > size.width - 1)
			bottomRight.x = size.width - 1;
		if (bottomRight.y > size.height - 1)
			bottomRight.y = size.height - 1;

		return new Rect(topLeft, bottomRight);
	}

	public static boolean lineIstWithinSegment(Line line, Segment segment, Size size) {
		Rect lineRect = Imgproc.boundingRect(line.getLinePoints());
		Rect segmentRect = calcExtendedRect(segment.getRegion(), size);

		if (segmentRect.contains(lineRect.tl()) && segmentRect.contains(lineRect.br())) {
			return true;
		}

		return false;
	}

	public static void includeLines(ArrayList<Segment> segments, ArrayList<Line> lines, Size size) {
		for (Line line : lines) {
			for (Segment segment : segments) {
				if (segment.getType().equals(line.getType()) && lineIstWithinSegment(line, segment, size)) {
					segment.addLine(line);
					break;
				}
			}
		}
	}

	public static ArrayList<Segment> importSegmentsAndLines(Mat image, String segmentsInputPath,
			String linesInputPath) {
		LinePage page = SegmentImport.readPageXML(segmentsInputPath);
		ArrayList<Line> lines = LineImport.readPageXML(linesInputPath);

		ArrayList<Segment> segments = page.getSegments();
		includeLines(segments, lines, page.getImageSize());
		
		return segments;
	}
}