package linePP.dataImport;

import java.io.File;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import linePP.data.Line;
import helper.TypeConverter;
import regions.type.RegionType;

public class LineImport {

	private static MatOfPoint extractPoints(Element coords) {
		String pointsString = coords.getAttribute("points");
		ArrayList<Point> pointList = new ArrayList<Point>();

		boolean finished = false;

		while (!finished) {
			int spacePosition = pointsString.indexOf(" ");

			if (spacePosition > 0) {
				String pointString = pointsString.substring(0, spacePosition);
				pointsString = pointsString.substring(spacePosition + 1);

				int x = Integer.parseInt(pointString.substring(0, pointString.indexOf(",")));
				int y = Integer.parseInt(pointString.substring(pointString.indexOf(",") + 1));

				Point newPoint = new Point(x, y);
				pointList.add(newPoint);
			} else {
				int x = Integer.parseInt(pointsString.substring(0, pointsString.indexOf(",")));
				int y = Integer.parseInt(pointsString.substring(pointsString.indexOf(",") + 1));

				Point newPoint = new Point(x, y);
				pointList.add(newPoint);

				finished = true;
			}
		}

		Point[] pointArray = new Point[pointList.size()];
		MatOfPoint points = new MatOfPoint(pointList.toArray(pointArray));

		return points;
	}

	private static Line extractLines(Node lineNode) {
		Element lineElement = (Element) lineNode;
		String typeString = lineElement.getAttribute("type");
		RegionType type = TypeConverter.stringToType(typeString);

		Element coords = (Element) lineElement.getElementsByTagName("Coords").item(0);
		MatOfPoint points = extractPoints(coords);

		Line line = new Line(points, null, type);

		if(line.getTopRight().x - line.getTopLeft().x < line.getBottomLeft().y - line.getTopLeft().y) {
			return null;
		}
		
		return line;
	}

	public static ArrayList<Line> readPageXML(String inputPath) {

		ArrayList<Line> lines = null;

		try {
			File inputFile = new File(inputPath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document document = dBuilder.parse(inputFile);
			document.getDocumentElement().normalize();

			// Element pageInfo = (Element)
			// document.getElementsByTagName("Page").item(0);
			// String imageFilename = pageInfo.getAttribute("imageFilename");
			NodeList textLines = document.getElementsByTagName("TextRegion");

			lines = new ArrayList<Line>();

			for (int i = 0; i < textLines.getLength(); i++) {
				Line newLine = extractLines(textLines.item(i));
				
				if(newLine != null) {
					lines.add(newLine);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Reading XML file failed!");
		}

		return lines;
	}
}