package linePP.dataImport;

import java.io.File;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Size;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import linePP.data.LinePage;
import linePP.data.Segment;
import helper.TypeConverter;
import regions.type.RegionType;

public class SegmentImport {

	private static MatOfPoint extractPoints(Element coords) {
		String pointsString = coords.getAttribute("points");
		ArrayList<Point> pointList = new ArrayList<Point>();

		boolean finished = false;

		while (!finished) {
			int spacePosition = pointsString.indexOf(" ");

			if (spacePosition > 0) {
				String pointString = pointsString.substring(0, spacePosition);
				pointsString = pointsString.substring(spacePosition + 1);

				int x = Integer.parseInt(pointString.substring(0, pointString.indexOf(",")));
				int y = Integer.parseInt(pointString.substring(pointString.indexOf(",") + 1));

				Point newPoint = new Point(x, y);
				pointList.add(newPoint);
			} else {
				int x = Integer.parseInt(pointsString.substring(0, pointsString.indexOf(",")));
				int y = Integer.parseInt(pointsString.substring(pointsString.indexOf(",") + 1));

				Point newPoint = new Point(x, y);
				pointList.add(newPoint);

				finished = true;
			}
		}

		Point[] pointArray = new Point[pointList.size()];
		MatOfPoint points = new MatOfPoint(pointList.toArray(pointArray));

		return points;
	}

	private static Segment extractSegments(Node segmentNode, boolean isTextSegment) {
		Element segmentElement = (Element) segmentNode;
		RegionType type = RegionType.image;

		if (isTextSegment) {
			String typeString = segmentElement.getAttribute("type");
			type = TypeConverter.stringToType(typeString);
		}

		Element coords = (Element) segmentElement.getElementsByTagName("Coords").item(0);
		MatOfPoint points = extractPoints(coords);

		Segment segment = new Segment(points, type);

		return segment;
	}

	public static LinePage readPageXML(String inputPath) {
		LinePage page = null;
		ArrayList<Segment> segments = new ArrayList<Segment>();

		try {
			File inputFile = new File(inputPath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document document = dBuilder.parse(inputFile);
			document.getDocumentElement().normalize();

			Element pageInfo = (Element) document.getElementsByTagName("Page").item(0);
			Size imageSize = new Size(Integer.parseInt(pageInfo.getAttribute("imageWidth")),
					Integer.parseInt(pageInfo.getAttribute("imageHeight")));
			
			NodeList textSegments = document.getElementsByTagName("TextRegion");
			NodeList imageSegments = document.getElementsByTagName("ImageRegion");

			for (int i = 0; i < textSegments.getLength(); i++) {
				Segment newSegment = extractSegments(textSegments.item(i), true);
				segments.add(newSegment);
			}

			for (int i = 0; i < imageSegments.getLength(); i++) {
				Segment newSegment = extractSegments(imageSegments.item(i), false);
				segments.add(newSegment);
			}
			
			page = new LinePage(inputPath, segments);
			page.setImageSize(imageSize);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Reading XML file failed!");
		}

		return page;
	}
}