package linePP.drawing;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import org.opencv.core.Mat;

import linePP.data.LinePage;
import linePP.helper.ImageProcessor;

@SuppressWarnings("serial")
public class BackgroundPanel extends JPanel {

	private LinePage page;
	private BufferedImage buffImage;
	private Mat matImage;
	private Painter painter;

	private LineRegionManager regionManager;

	public BackgroundPanel(LinePage page, LineRegionManager regionManager) {
		super();
		setPainter(new Painter(this));
		setRegionManager(regionManager);

		addListeners();

		updateImage(page);
		setOpaque(false);
	}

	public void addListeners() {
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				painter.handleMousePressed(e);
				repaint();
			}

			public void mouseReleased(MouseEvent e) {
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
			}

			public void mouseMoved(MouseEvent e) {
				painter.handleMouseMoved(e);
				repaint();
			}
		});
	}

	public void paint(Graphics g) {
		super.paint(g);
		painter.paint(g);
	}

	public void updateImage(LinePage page) {
		setPage(page);

		Mat image = page.getResized();
		setMatImage(image);

		BufferedImage inputBI = ImageProcessor.matToBufferedImage(image);
		setBuffImage(inputBI);

		setPreferredSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));
		setSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));

		setRegionManager(new LineRegionManager(page));
		painter.setRegionManager(regionManager);
		painter.fullReset();

		repaint();
	}

	public LinePage getPage() {
		return page;
	}

	public void setPage(LinePage page) {
		this.page = page;
	}

	public BufferedImage getBuffImage() {
		return buffImage;
	}

	public void setBuffImage(BufferedImage buffImage) {
		this.buffImage = buffImage;
	}

	public Mat getMatImage() {
		return matImage;
	}

	public void setMatImage(Mat matImage) {
		this.matImage = matImage;
	}

	public Painter getPainter() {
		return painter;
	}

	public void setPainter(Painter painter) {
		this.painter = painter;
	}

	public LineRegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(LineRegionManager regionManager) {
		this.regionManager = regionManager;
		painter.setRegionManager(regionManager);
	}
}