package linePP.drawing;

import java.util.ArrayList;

import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.imgproc.Imgproc;

import linePP.data.LinePage;
import linePP.data.Segment;
import regions.colors.RegionColor;
import regions.colors.RegionColors;

public class LineRegionManager {

	private LinePage page;
	private ArrayList<RegionColor> regionColors;

	public LineRegionManager(LinePage page) {
		initPage(page);
		setRegionColors(RegionColors.getRegionColors());
	}

	public Segment identifyClickedSegment(Point point) {
		for (Segment segment : page.getSegments()) {
			if (Imgproc.pointPolygonTest(new MatOfPoint2f(segment.getResizedRegion().toArray()), point, false) > 0) {
				return segment;
			}
		}

		return null;
	}

	private void initPage(LinePage page) {
		for (Segment segment : page.getSegments()) {
			segment.resizeAll(page.getScaleFactor());
		}

		setPage(page);
	}

	public LinePage getPage() {
		return page;
	}

	public void setPage(LinePage page) {
		this.page = page;
	}

	public ArrayList<RegionColor> getRegionColors() {
		return regionColors;
	}

	public void setRegionColors(ArrayList<RegionColor> regionColors) {
		this.regionColors = regionColors;
	}
}