package linePP.edit;

import java.io.File;
import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.highgui.Highgui;

import dataManagement.Page;
import importExport.PageXMLWriter;
import linePP.GuiLinePP;
import linePP.auto.GuiLinePPAuto;
import linePP.data.LinePage;
import linePP.data.Segment;
import linePP.dataImport.Import;

public class LineEditor {

	public LineEditor(Page page, int verticalResolution, boolean auto) {
		// TODO
		check(page, verticalResolution, auto);
	}

	public boolean check(Page page, int verticalResolution, boolean auto) {
		String imagePath = page.getImagePath();
		String baseFolder = imagePath.substring(0, imagePath.lastIndexOf(page.getFileName()));
		String segmentFolderPath = baseFolder + "SegmentXMLs" + File.separator;
		String segmentPath = segmentFolderPath + page.getFileName() + ".xml";
		String linePath = segmentPath.replace("SegmentXMLs", "LineXMLs");

		if (page.getSegmentationResult() != null) {
			initOutputFolder(segmentFolderPath);
			PageXMLWriter.writePageXML(page, baseFolder, true);
		}

		// System.out.println("imagePath: " + imagePath);
		// System.out.println("baseFolder: " + baseFolder);
		// System.out.println("segmentPath: " + segmentPath);
		// System.out.println("linePath: " + linePath);

		File segmentFile = new File(segmentPath);
		File lineFile = new File(linePath);

		if (!segmentFile.exists()) {
			new DiaWarningLinePP(false);

			return false;
		} else if (!lineFile.exists()) {
			new DiaWarningLinePP(true);

			return false;
		} else if (segmentFile.exists() && lineFile.exists()) {
			Mat image = Highgui.imread(imagePath);
			ArrayList<Segment> segments = Import.importSegmentsAndLines(image, segmentPath, linePath);

			LinePage linePage = new LinePage(imagePath, segments);
			linePage.init(verticalResolution);

			if (!auto) {
				GuiLinePP gui = new GuiLinePP(linePage, page, "", verticalResolution);
				gui.setVisible(true);
			} else {
				GuiLinePPAuto gui = new GuiLinePPAuto(linePage, page, "", verticalResolution);
				gui.setVisible(true);
			}
		}

		return false;
	}

	private void initOutputFolder(String inputFolder) {
		if (!inputFolder.endsWith(File.separator)) {
			inputFolder += File.separator;
		}

		new File(inputFolder).mkdirs();
	}
}