package linePP.pageXML;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import linePP.calc.Page;
import linePP.calc.Segment;

public class Export {
	private static void addPoints(Document document, Element coordsElement, MatOfPoint pointMat) {
		Point[] points = pointMat.toArray();
		String pointCoords = "";

		for (int i = 0; i < points.length; i++) {
			pointCoords += (int) points[i].x + "," + (int) points[i].y + " ";
		}

		if(pointCoords.length() > 0) {
			pointCoords = pointCoords.substring(0, pointCoords.length() - 1);
		}
		coordsElement.setAttribute("points", pointCoords);
	}

	private static void addRegions(Document document, Element pageElement, ArrayList<Segment> segments) {
		int regionCnt = 0;

		for (Segment segment : segments) {
			String regionType = segment.getType();
			regionType = regionType.replace("_", "-");

			Element regionElement = document.createElement("TextRegion");
			regionElement.setAttribute("type", regionType);
			regionElement.setAttribute("id", "r" + regionCnt);

			Element coordsElement = document.createElement("Coords");
			addPoints(document, coordsElement, segment.getPolygon());

			regionElement.appendChild(coordsElement);
			pageElement.appendChild(regionElement);
			regionCnt++;
		}
	}

	public static void writePageXML(Page page, ArrayList<Segment> segments, String outputFolder) {
		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document document = docBuilder.newDocument();

			Element rootElement = document.createElement("PcGts");

			rootElement.setAttribute("xmlns", "http://schema.primaresearch.org/PAGE/gts/pagecontent/2013-07-15");
			rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			rootElement.setAttribute("xsi:schemaLocation",
					"http://schema.primaresearch.org/PAGE/gts/pagecontent/2013-07-15 http://schema.primaresearch.org/PAGE/gts/pagecontent/2013-07-15/pagecontent.xsd");

			document.appendChild(rootElement);

			Element metadataElement = document.createElement("Metadata");
			Element creatorElement = document.createElement("Creator");
			Element createdElement = document.createElement("Created");
			Element changedElement = document.createElement("LastChange");

			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
			String date = sdf.format(new Date());

			Node creatorTextNode = document.createTextNode("Christian Reul");
			Node createdTextNode = document.createTextNode(date);
			Node changedTextNode = document.createTextNode(date);

			creatorElement.appendChild(creatorTextNode);
			createdElement.appendChild(createdTextNode);
			changedElement.appendChild(changedTextNode);

			metadataElement.appendChild(creatorElement);
			metadataElement.appendChild(createdElement);
			metadataElement.appendChild(changedElement);

			rootElement.appendChild(metadataElement);

			Element pageElement = document.createElement("Page");
			pageElement.setAttribute("imageFilename", page.getImageFilename());
			pageElement.setAttribute("imageWidth", "" + page.getMatImage().width());
			pageElement.setAttribute("imageHeight", "" + page.getMatImage().height());
			rootElement.appendChild(pageElement);

			addRegions(document, pageElement, segments);

			// write content into xml file
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(document);

			if(!outputFolder.endsWith("/")) {
				outputFolder += "/";
			}
			
			String outputPath = outputFolder + page.getImageFilename() + ".xml";

			StreamResult result = new StreamResult(new File(outputPath));
			transformer.transform(source, result);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}